module Glossync
  DEFAULT_CONFIG = {
    reload_backend:              false,
    write_fallback_result:       false,
    missing_translation_handler: MissingTranslationHandler::Default.new,
    base_locale:                 :en,
  }.freeze

  # define convinence methods for each of the configuration paramaters special
  # methods added for parameters that take booleans, e.g. dont_xxxx or xxxx!
  DEFAULT_CONFIG.each do |k, v|
    (case v
    when true, false
      [
        [:"dont_#{k}", -> { @options[k] = false }],
        [:"#{k}!",     -> { @options[k] = true } ],
        [:"#{k}?",     -> { @options[k] }        ],
      ]
    else
      []
    end + [
      [:"#{k}=", ->(v) { @options[k] = v }],
      [:"#{k}",  ->    { @options[k]}     ]
    ]).each { |meth, body| Glossync.define_singleton_method(meth, &body) }
  end

  class << self
    # A hash which stores all of Glossync's global options.
    attr_reader :options

    # If called with a block, executes the block setting the argument to Glossync,
    # else returns the options hash
    def config(&block)
      @options ||= DEFAULT_CONFIG.dup
      block_given? ? block.call(Glossync) : @options
    end
  end
end

